@@ -464,6 +464,10 @@
         <source>Use a custom data directory:</source>
         <translation>Izmantot pielāgotu datu mapi:</translation>
     </message>
+    <message>
+        <source>Bitcoin</source>
+        <translation>Bitcoin</translation>
+    </message>
     <message>
         <source>Error</source>
         <translation>Kļūda</translation>
@@ -521,10 +525,6 @@
         <source>Number of script &amp;verification threads</source>
         <translation>Skriptu &amp;pārbaudes pavedienu skaits</translation>
     </message>
-    <message>
-        <source>Allow incoming connections</source>
-        <translation>Atļaut ienākošos savienojumus</translation>
-    </message>
     <message>
         <source>IP address of the proxy (e.g. IPv4: 127.0.0.1 / IPv6: ::1)</source>
         <translation>Starpniekservera IP adrese (piem. IPv4: 127.0.0.1 / IPv6: ::1)</translation>
@@ -533,10 +533,6 @@
         <source>Minimize instead of exit the application when the window is closed. When this option is enabled, the application will be closed only after selecting Exit in the menu.</source>
         <translation>Minimizēt nevis aizvērt aplikāciju, kad logs tiek aizvērts. Kad šī iespēja ir ieslēgta, aplikācija tiks aizvērta, izvēloties Aizvērt izvēlnē.</translation>
     </message>
-    <message>
-        <source>Third party transaction URLs</source>
-        <translation>Trešo personu transakciju URLs</translation>
-    </message>
     <message>
         <source>Active command-line options that override above options:</source>
         <translation>Aktīvās komandrindas opcijas, kuras pārspēko šos iestatījumus:</translation>
@@ -645,6 +641,10 @@
         <source>Confirm options reset</source>
         <translation>Apstiprināt iestatījumu atiestatīšanu</translation>
     </message>
+    <message>
+        <source>Error</source>
+        <translation>Kļūda</translation>
+    </message>
     <message>
         <source>The supplied proxy address is invalid.</source>
         <translation>Norādītā starpniekservera adrese nav derīga.</translation>
@@ -717,7 +717,27 @@
         <source>%1 and %2</source>
         <translation>%1 un %2</translation>
     </message>
-    </context>
+    <message>
+        <source>%1 B</source>
+        <translation>%1 B</translation>
+    </message>
+    <message>
+        <source>%1 KB</source>
+        <translation>%1 KB</translation>
+    </message>
+    <message>
+        <source>%1 MB</source>
+        <translation>%1 MB</translation>
+    </message>
+    <message>
+        <source>%1 GB</source>
+        <translation>%1 GB</translation>
+    </message>
+    <message>
+        <source>unknown</source>
+        <translation>nav zināms</translation>
+    </message>
+</context>
 <context>
     <name>QObject::QObject</name>
     </context>
@@ -786,10 +806,6 @@
         <source>&amp;Network Traffic</source>
         <translation>&amp;Tīkla Satiksme</translation>
     </message>
-    <message>
-        <source>&amp;Clear</source>
-        <translation>&amp;Notīrīt</translation>
-    </message>
     <message>
         <source>Totals</source>
         <translation>Kopsummas</translation>
@@ -810,30 +826,6 @@
         <source>Clear console</source>
         <translation>Notīrīt konsoli</translation>
     </message>
-    <message>
-        <source>Use up and down arrows to navigate history, and &lt;b&gt;Ctrl-L&lt;/b&gt; to clear screen.</source>
-        <translation>Izmantojiet bultiņas uz augšu un leju, lai pārvietotos pa vēsturi, un &lt;b&gt;Ctrl-L&lt;/b&gt; ekrāna notīrīšanai.</translation>
-    </message>
-    <message>
-        <source>Type &lt;b&gt;help&lt;/b&gt; for an overview of available commands.</source>
-        <translation>Ierakstiet &lt;b&gt;help&lt;/b&gt; lai iegūtu pieejamo komandu sarakstu.</translation>
-    </message>
-    <message>
-        <source>%1 B</source>
-        <translation>%1 B</translation>
-    </message>
-    <message>
-        <source>%1 KB</source>
-        <translation>%1 KB</translation>
-    </message>
-    <message>
-        <source>%1 MB</source>
-        <translation>%1 MB</translation>
-    </message>
-    <message>
-        <source>%1 GB</source>
-        <translation>%1 GB</translation>
-    </message>
     </context>
 <context>
     <name>ReceiveCoinsDialog</name>
@@ -849,10 +841,6 @@
         <source>&amp;Message:</source>
         <translation>&amp;Ziņojums:</translation>
     </message>
-    <message>
-        <source>R&amp;euse an existing receiving address (not recommended)</source>
-        <translation>&amp;Atkārtoti izmantot esošo saņemšanas adresi (nav ieteicams)</translation>
-    </message>
     <message>
         <source>Clear all fields of the form.</source>
         <translation>Notīrīt visus laukus formā.</translation>
@@ -1263,10 +1251,6 @@
         <source>Verifying blocks...</source>
         <translation>Pārbauda blokus...</translation>
     </message>
-    <message>
-        <source>Verifying wallet...</source>
-        <translation>Pārbauda maciņu...</translation>
-    </message>
     <message>
         <source>Wallet options:</source>
         <translation>Maciņa iespējas:</translation>
@@ -1315,14 +1299,6 @@
         <source>Allow DNS lookups for -addnode, -seednode and -connect</source>
         <translation>Atļaut DNS uzmeklēšanu priekš -addnode, -seednode un -connect</translation>
     </message>
-    <message>
-        <source>Loading addresses...</source>
-        <translation>Ielādē adreses...</translation>
-    </message>
-    <message>
-        <source>Invalid -proxy address: '%s'</source>
-        <translation>Nederīga -proxy adrese: '%s'</translation>
-    </message>
     <message>
         <source>Unknown network specified in -onlynet: '%s'</source>
         <translation>-onlynet komandā norādīts nepazīstams tīkls: '%s'</translation>
@@ -1335,10 +1311,6 @@
         <source>Loading block index...</source>
         <translation>Ielādē bloku indeksu...</translation>
     </message>
-    <message>
-        <source>Add a node to connect to and attempt to keep the connection open</source>
-        <translation>Pievienot mezglu, kam pievienoties un turēt savienojumu atvērtu</translation>
-    </message>
     <message>
         <source>Loading wallet...</source>
         <translation>Ielādē maciņu...</translation>
@@ -1347,10 +1319,6 @@
         <source>Cannot downgrade wallet</source>
         <translation>Nevar maciņa formātu padarīt vecāku</translation>
     </message>
-    <message>
-        <source>Cannot write default address</source>
-        <translation>Nevar ierakstīt adresi pēc noklusēšanas</translation>
-    </message>
     <message>
         <source>Rescanning...</source>
         <translation>Skanēju no jauna...</translation>
